from typing import Literal, Tuple
from datetime import date

from sqlalchemy import select, update
from sqlalchemy import ScalarResult

import ulid

from ..database.db import async_session
from ..database.models import UserAccount, UserProfile, Contact


async def select_user_by(condition: Literal['email', 'username', 'id'], value: str) -> Tuple[bool, UserAccount]:
    session = async_session()
    res: ScalarResult[UserAccount] = ScalarResult[UserAccount]
    match condition:
        case 'email':
            res = await session.scalars(select(UserAccount).where(UserAccount.email == value))
        case 'username':
            res = await session.scalars(select(UserAccount).where(UserAccount.username == value))
        case 'id':
            res = await session.scalars(select(UserAccount).where(UserAccount.id == value))

    user = res.first()

    return (True, user) if user else (False, None)


async def insert_user(username: str, password: str, email: str):
    session = async_session()
    id = ulid.new().str
    user = UserAccount(id=id, username=username, password=password, email=email)
    profile = UserProfile(nickname=username)
    contact = Contact(
        friends={id: {'friendRemark': None, 'friendGroup': '我的好友'}}, friend_groups=['我的好友'],
        group_chats={}
    )
    user.profile = profile
    user.contact = contact
    session.add(user)
    await session.commit()
    await session.close()


async def select_profile(id: str) -> UserProfile:
    session = async_session()
    res: ScalarResult[UserProfile] = await session.scalars(select(UserProfile).where(UserProfile.user_id == id))
    res: UserProfile = res.first()
    await session.close()
    return res


async def update_profile_basic(id: str, nickname: str, location: str, birthday: str, gender: str, ):
    session = async_session()
    await session.execute(
        update(UserProfile)
        .where(UserProfile.user_id == id)
        .values(nickname=nickname, location=location, birthday=date.fromisoformat(birthday), gender=gender)
    )
    await session.commit()
    await session.close()


async def update_profile_sign(id: str, sign: str):
    session = async_session()
    await session.execute(
        update(UserProfile)
        .where(UserProfile.user_id == id)
        .values(sign=sign)
    )
    await session.commit()
    await session.close()


async def update_profile_avatar(id: str, avatar_name: str):
    session = async_session()
    await session.execute(
        update(UserProfile)
        .where(UserProfile.user_id == id)
        .values(avatar=avatar_name)
    )
    await session.commit()
    await session.close()


async def update_account(account: Literal['username', 'email', 'password'], id: str, value: str):
    session = async_session()
    match account:
        case 'username':
            await session.execute(
                update(UserAccount)
                .where(UserAccount.id == id)
                .values(username=value)
            )
        case 'email':
            await session.execute(
                update(UserAccount)
                .where(UserAccount.id == id)
                .values(email=value)
            )
        case 'password':
            await session.execute(
                update(UserAccount)
                .where(UserAccount.id == id)
                .values(password=value)
            )
    await session.commit()
    await session.close()
