from datetime import datetime, date
from enum import StrEnum

from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy.orm import DeclarativeBase, relationship
from sqlalchemy.orm import mapped_column, Mapped
from sqlalchemy import String, Integer, Enum, DateTime, ARRAY, Date
from sqlalchemy import ForeignKey


class Base(DeclarativeBase):
    pass


class UserAccount(Base):
    __tablename__ = "user_account"

    id: Mapped[str] = mapped_column(String(26), primary_key=True)
    username: Mapped[str] = mapped_column(String(20), unique=True)
    email: Mapped[str] = mapped_column(String, unique=True)
    password: Mapped[str] = mapped_column(String(60))
    updated_at: Mapped[datetime] = mapped_column(default=datetime.now)
    created_at: Mapped[datetime] = mapped_column(default=datetime.now, onupdate=datetime.now)
    profile: Mapped['UserProfile'] = relationship(back_populates='user')
    contact: Mapped['Contact'] = relationship(back_populates='user')

    def __repr__(self):
        return f'UserAccount(username={self.username}, email={self.email})'

    def to_dict(self) -> dict:
        return {
            'id': self.id,
            'username': self.username,
            'email': self.email,
        }


class Gender(StrEnum):
    man = 'man'
    woman = 'woman'


class UserProfile(Base):
    __tablename__ = 'user_profile'

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    nickname: Mapped[str] = mapped_column(String)
    gender: Mapped[Gender] = mapped_column(Enum(Gender), nullable=True)
    birthday: Mapped[date] = mapped_column(Date, nullable=True)
    location: Mapped[str] = mapped_column(String, nullable=True)
    status: Mapped[str] = mapped_column(String, nullable=True)
    sign: Mapped[str] = mapped_column(String, nullable=True)
    avatar: Mapped[str] = mapped_column(String, nullable=True)
    user_id: Mapped[str] = mapped_column(ForeignKey('user_account.id', ondelete='CASCADE'))
    user: Mapped['UserAccount'] = relationship(back_populates='profile')

    def __repr__(self):
        return f'UserProfile(user={self.user_id},' \
               f'nickname={self.nickname})'

    def to_dict_all(self):
        return {
            'nickname': self.nickname,
            'gender': self.gender,
            'birthday': self.birthday.isoformat(),
            'location': self.location,
            'status': self.status,
            'sign': self.sign,
            'avatar': self.avatar,
        }


class Contact(Base):
    __tablename__ = 'contact'

    id: Mapped[int] = mapped_column(Integer, primary_key=True)
    friends: Mapped[dict] = mapped_column(JSONB, nullable=True)
    friend_groups: Mapped[list[str]] = mapped_column(ARRAY(String), nullable=True)
    group_chats: Mapped[dict] = mapped_column(JSONB, nullable=True)
    user_id: Mapped[str] = mapped_column(ForeignKey('user_account.id', ondelete='CASCADE'))
    user: Mapped['UserAccount'] = relationship(back_populates='contact')